package ss2011.se.dp.filesystem;

import java.io.File;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import ss2011.se.dp.visitors.CountFilesVisitor;
import ss2011.se.dp.visitors.RegularExpressionVisitor;
import ss2011.se.dp.visitors.SummarizeFileSizeVisitor;
import ss2011.se.dp.visitors.Visitor;

public abstract class AbstractFile
{

    private AbstractFile parent;
    private Set<AbstractFile> children = new HashSet<AbstractFile>();
    private String name;

    public AbstractFile(AbstractFile parent, String name)
    {
        this.parent = parent;
        this.name = name;
    }

    public Set<AbstractFile> getChildren()
    {
        return Collections.unmodifiableSet(children);
    }

    public AbstractFile getParent()
    {
        return parent;
    }

    public abstract long getSize();

    public boolean addChild(AbstractFile file)
    {
        return children.add(file);
    }

    public String getName()
    {
        return name;
    }

    public long countDotJavaSize()
    {
        long count = name.toLowerCase().endsWith(".java") ? getSize() : 0;
        for (AbstractFile f : children)
        {
            count += f.countDotJavaSize();
        }
        return count;
    }

    public long countDotClass()
    {
        long count = name.toLowerCase().endsWith(".class") ? 1 : 0;
        for (AbstractFile f : children)
        {
            count += f.countDotClass();
        }
        return count;
    }
    
    public long countAllFiles()
    {
        long count = 1;
        for (AbstractFile f : children)
        {
            count += f.countAllFiles();
        }
        return count;
    }
	
    public long countTotalFileSize()
    {
        long count = getSize();
        for (AbstractFile f : children)
        {
            count += f.countTotalFileSize();
        }
        return count;
    }
    
    public AbstractFile createFile(AbstractFile parent, File file)
    {
        if (!file.isDirectory())
        {
            if (file.getName().toLowerCase().endsWith(".jar"))
            {
                return new JARArchive(parent, file);
            }
            if (file.getName().toLowerCase().endsWith(".zip"))
            {
                return new ZIPArchive(parent, file);
            } else
            {
                return new RegularFile(parent, file.getName(), file.length());
            }
        } else
        {
            return new Directory(parent, file);
        }
    }

    public void printTree()
    {
        String path = getName();
        AbstractFile parent = getParent();
        while (parent != null)
        {
            path = parent.getName() + "/" + path;
            parent = parent.getParent();
        }
        System.out.println(path);
        for (AbstractFile file : getChildren())
        {
            file.printTree();
        }
    }
    
    public void accept(Visitor visitor) {
        
    }

}
